#include "hidcontrol.h"
#include "ui_hidcontrol.h"
#include <QMessageBox>
#include <QTextStream>
#include <iomanip>
#include <sstream>
#include "usbconfig.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "hiddata.h"

HIDControl::HIDControl(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::HIDControl)
{
    ui->setupUi(this);
    connect(ui->readDataButton,SIGNAL(clicked()),this,SLOT(readData()));
    connect(ui->writeDataButton,SIGNAL(clicked()),this,SLOT(writeData()));
    // connect(&timer, SIGNAL(timeout()), this, SLOT(readData()));
    // timer.start(500);
}


char* HIDControl::usbErrorMessage(int errCode) {
    static char buffer[80];

    switch (errCode) {
    case USBOPEN_ERR_ACCESS:
        return "Access to device denied";
    case USBOPEN_ERR_NOTFOUND:
        return "The specified device was not found";
    case USBOPEN_ERR_IO:
        return "Communication error with device";
    default:
        sprintf(buffer, "Unknown USB error %d", errCode);
        return buffer;
    }
    return NULL; /* not reached */
}

usbDevice_t* HIDControl::openDevice(void) {
    usbDevice_t *dev = NULL;
    unsigned char rawVid[2] = { USB_CFG_VENDOR_ID }, rawPid[2] = {
            USB_CFG_DEVICE_ID };
    char vendorName[] = { USB_CFG_VENDOR_NAME, 0 }, productName[] = {
            USB_CFG_DEVICE_NAME, 0 };
    int vid = rawVid[0] + 256 * rawVid[1];
    int pid = rawPid[0] + 256 * rawPid[1];
    int err;

    if ((err = usbhidOpenDevice(&dev, vid, vendorName, pid, productName, 0))
            != 0) {
        fprintf(stderr, "error finding %s: %s\n", productName,
                usbErrorMessage(err));
        return NULL;
    }
    return dev;
}

HIDControl::~HIDControl()
{
    delete ui;
}

void HIDControl::readData(){

    int err;
    QMessageBox box;

    if ((dev = openDevice()) == NULL) {
//        box.setText("Unable to open device");
//        box.exec();
        return;
    }
    int len = sizeof(buffer);
    if ((err = usbhidGetReport(dev, 0, buffer, &len)) != 0) {
/*        QString message(usbErrorMessage(err));
        box.setText(message);
        box.exec()*/;
        return;
    }
    else { // output buffer to textbox

        // QString out = hexDump(buffer,1,128);

        QString out;

        for (int i = 1;i < len;i++) {

            QString num = QString::number(buffer[i],16).rightJustified(2,'0');

            if (num.length() > 2) {
                num = num.right(2);
            }
            out.append(num);

            if (i % 16 == 0) {
                out.append("\n");
            }
            else {
                out.append(" ");
            }

        }


        ui->readDataTextEdit->setText(out);
    }

    usbhidCloseDevice(dev);


}

QString HIDControl::hexDump(char* data, int offset,int length) {

    std::stringstream ts;

    int line = 0;
    int j = 0;
    unsigned char ascii[16];

    ts << std::setw(4) << std::setfill('0') << std::hex << static_cast<unsigned int> (line+offset) << " : ";

    for (int i = offset; i < length + offset; i++) {

        ts << std::setw(2) << std::setfill('0') << std::hex << static_cast<unsigned int> (data[i]) << " ";

        ascii[j++] = static_cast<unsigned char> (data[i]);

        if ((i + 1 + offset) % 16 == 0) {
            ts << " | ";

            for (int k = 0; k < 16; k++) {
                if ((ascii[k] >= 0x20) && (ascii[k] < 0x7f))
                    ts << ascii[k];
                else
                    ts << ".";
            }

            j = 0;

            ts << "\n";
            line++;
            // if ((i + 1 + offset) < length)
            ts << std::setw(4) << std::setfill('0') << std::hex << static_cast<unsigned int> (line * 16 + offset) << " : ";
        }

    }
    ts << "\n";
    return QString::fromStdString(ts.str());

}

void HIDControl::writeData() {

    int err;
    QMessageBox box;
    if ((dev = openDevice()) == NULL) {
        box.setText("Unable to open device");
        box.exec();
        return;
    }

    memset(buffer, 0, sizeof(buffer));

    QString out = ui->writeDataTextEdit->toPlainText();
    QStringList list = out.split(" ");

    int i=1;

    foreach(QString s,list)
        buffer[i++] = s.toInt();


    if ((err = usbhidSetReport(dev, buffer, sizeof(buffer))) != 0) { /* add a dummy report ID */
        QString message(usbErrorMessage(err));
        box.setText(message);
        box.exec();
        return;
    }
    else { // output buffer to device
        ui->writeDataTextEdit->setText("write ok.");
    }

    usbhidCloseDevice(dev);
}
